#include "mainwindow.h"
#include "ui_mainwindow.h"
#include "qfiledialog.h"
#include "qtextstream.h"
#include "qstring.h"
#include "qvector.h"
#include "qstringlist.h"
#include "qmessagebox.h"
#include "qvarlengtharray.h"

QString input_filename;
QString output_filename;

int input_file_selected = 0;



MainWindow::MainWindow(QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::MainWindow)
{
    ui->setupUi(this);
}

MainWindow::~MainWindow()
{
    delete ui;
}

void MainWindow::on_Open_file_clicked()
{
    input_filename = QFileDialog::getOpenFileName(this, tr("Open File"), "", tr("Files (*.gcode;*.txt*)"));
    ui->Active_file->setText(input_filename);
    input_file_selected = 1;
}

void MainWindow::on_Convert_file_clicked()
{
    if (input_file_selected == 0) //check if a file has been selected
    {
        QMessageBox nofileavailable;
        nofileavailable.setText("No File is selected");
        nofileavailable.exec();
    }
    else
    {
        //create write file

        QFile OutputFile;
        //comment to make debugging easier
        output_filename = QFileDialog::getSaveFileName(this, tr("Save File"), "", tr("Files (*.txt)"));//select a place to create the file
        OutputFile.setFileName(output_filename); //set the name of the file
        //clear file if it already exists

        OutputFile.open(QIODevice::WriteOnly | QIODevice::Text | QIODevice::Truncate); //create the file
        QTextStream output_data(&OutputFile); //create file for output



        //create grid parameters
        //(number of grid lines is in Y direction, length of those lines is in X direction)

        // * number of lines
        float temp_grid_number_of_lines = ((ui->DPI_spinbox->value() * ui->Y_size_spinbox->value())/25.4);
        int grid_number_of_lines = int(temp_grid_number_of_lines);
        //convert number of lines to a multiple of the amount of sweeps
        grid_number_of_lines /= ui->nozzle_rows_spinbox->value();
        grid_number_of_lines ++; //add one to round up (excess lines will be ignored further in the code)
        int grid_number_of_single_sweeps = (grid_number_of_lines); //number of single sweeps to help calculate the grid shape
        grid_number_of_lines *= ui->nozzle_rows_spinbox->value();

        //multiply the grid by the number of sweeps
        grid_number_of_lines *= (ui->Sweepin_directions_2->currentIndex()+1);

        // * step size based on the amount of sweeps and the DPI
        float standard_step_size = 25.4 / ui->DPI_spinbox->value();
        float sweeping_step_size = standard_step_size / (ui->Sweepin_directions_2->currentIndex()+1);

        // * coordinates and direction of every line
        float *grid_line_coordinate = new float[grid_number_of_lines];
        int *grid_line_direction = new int[grid_number_of_lines];

        int temp_grid_nozzle_position = 0;
        int direction_toggle = 1;
        for (int s = 0; s < grid_number_of_single_sweeps; s++)//the amount of total sweeps
        {
            for (int d = 0; d < (ui->Sweepin_directions_2->currentIndex()+1); d++) //the amount of passes per sweep
            {
                for (int n = 0; n < ui->nozzle_rows_spinbox->value(); n ++) //the amount of nozzles
                {
                    float temp_coor_calc;
                    temp_coor_calc = (s*ui->nozzle_rows_spinbox->value())+n; // the amout of nozzles between 0 and here
                    temp_coor_calc *= standard_step_size; //multiply by standard step size
                    temp_coor_calc += (sweeping_step_size*d); //offset for double, tripple of quadruple sweeps
                    grid_line_coordinate[temp_grid_nozzle_position] = temp_coor_calc; // add to coordinate buffer
                    grid_line_direction[temp_grid_nozzle_position] = direction_toggle;
                    temp_grid_nozzle_position ++;
                }
                if (ui->Sweepin_directions->currentIndex() == 1)//double direction sweeping
                {
                    if (direction_toggle == 1) //toggle between positive and negative after every sweep
                    {
                        direction_toggle = 0;
                    }
                    else
                    {
                        direction_toggle = 1;
                    }
                }
            }
        }




        //open file
        QFile Inputfile(input_filename);
        Inputfile.open(QIODevice::ReadOnly);
        QTextStream readtextStream(&Inputfile);




        float *starting_coordinates = new float[5];
        float *coordinate_history = new float[5];
        int start_of_file = 1;
        int printing_shape = 0;

        QVector<float> buffer_X_coor(0);
        QVector<int> buffer_Y_coor(0);
        float Z_height = 0;

        int end_of_file = 0;
        while (end_of_file == 0)
        {
            //read file, line by line. Mark start point of a line
            QString read_line = readtextStream.readLine();
            QStringList decode_line = read_line.split(" ", QString::SkipEmptyParts);
            int stopping_point = decode_line.size();
            int character_reading = 0;
            int *conversion_num_buffer = new int[3];
            float *conversion_coor_buffer = new float[5];
            //make all buffers -1
            for (int n = 0; n < 3; n++)
            {
                conversion_num_buffer[n] = -1;
            }
            for (int c = 0; c < 5; c++)
            {
                conversion_coor_buffer[c] = -1.00;
            }

            //while a line is being read.
            while (stopping_point > 0)
            {
                //decode line
                if (character_reading == 0)
                {
                    if (decode_line[character_reading].startsWith("G"))
                    {
                        decode_line[character_reading].remove(0,1); //remove first character
                        if (decode_line[character_reading].toInt())
                        {
                            //move to num_buffer
                            conversion_num_buffer[0] = decode_line[character_reading].toInt();
                        }
                        else
                        {
                            stopping_point = 0; //no readable line, break from decoding
                        }
                    }
                    else
                    {
                        stopping_point = 0; //no readable line, break from decoding
                    }
                }
                else
                {
                    int read_character = 0;
                    if (decode_line[character_reading].startsWith("X"))
                    {
                        read_character = 1;
                    }
                    else if (decode_line[character_reading].startsWith("Y"))
                    {
                        read_character = 2;
                    }
                    else if (decode_line[character_reading].startsWith("Z"))
                    {
                        read_character = 3;
                    }
                    else if (decode_line[character_reading].startsWith("E"))
                    {
                        read_character = 4;
                    }
                    else if (decode_line[character_reading].startsWith("F"))
                    {
                        read_character = 5;
                    }
                    if (read_character > 0)
                    {
                        //decode line
                        decode_line[character_reading].remove(0,1); //remove first character
                        //if (decode_line[character_reading].toFloat())
                        //{
                        //move to coor_buffer
                        conversion_coor_buffer[read_character-1] = decode_line[character_reading].toFloat();
                        //-1 protection
                        if (conversion_coor_buffer[read_character-1] == -1.00)
                        {
                            conversion_coor_buffer[read_character-1] = float(-0.99);
                        }
                        //}
                        /*else
                            {
                                stopping_point = 0; //no readable line, break from decoding
                            }*/
                    }
                    else
                    {
                        stopping_point = 0; //no readable line, break from decoding
                    }
                }

                //put line in buffer

                stopping_point--;
                character_reading++;
            }

            //constantly calculate intersections and add toggle points at intersections
            if (start_of_file == 1)
            {
                //mark first G1 coordinates and put first coordinates in history
                if (conversion_num_buffer[0] == 1) //G1
                {
                    start_of_file = 0;
                    for (int c = 0; c < 5; c++)
                    {
                        if (conversion_coor_buffer[c] != -1)
                        {
                            //starting_coordinates[c] = conversion_coor_buffer[c];
                            coordinate_history[c] = conversion_coor_buffer[c];
                            Z_height = conversion_coor_buffer[2];
                        }
                    }
                }
                else if (conversion_num_buffer[0] == 92) //G92, don't start print, but do add to history
                {
                    for (int c = 0; c < 5; c++)
                    {
                        if (conversion_coor_buffer[c] != -1)
                        {
                            coordinate_history[c] = conversion_coor_buffer[c];
                        }
                    }
                }
            }
            else //normal decode of the line
            {
                //if Z- has changed, new layer procedure
                if (conversion_coor_buffer[2] > Z_height)
                {

                    //reorder vector
                    for (int l = 0; l < (grid_number_of_lines/ui->nozzle_rows_spinbox->value()); l++)
                    {
                        //--create lines read integer
                        //--for loop through the file, marking multiples of nozzlecount, ie. 0-11, 12-23 etc.
                        int value_read = 0;
                        int lower_limit = l * ui->nozzle_rows_spinbox->value();
                        int upper_limit = lower_limit + ui->nozzle_rows_spinbox->value();
                        QVector<float> X_temp_list(0);
                        QVector<int> Y_temp_list(0);
                        for (int n = 0; n < buffer_Y_coor.size(); n++)
                        {
                            if (buffer_Y_coor[n] >= lower_limit && buffer_Y_coor[n] < upper_limit)  //--when found, move these to a new vector
                            {
                                value_read = 1;
                                X_temp_list.append(buffer_X_coor[n]);
                                Y_temp_list.append(buffer_Y_coor[n]);
                            }
                        }
                        if (value_read == 1) //if a value has been read, convert line
                        {
                            long sort_size = X_temp_list.size() - 1;
                            int sorted = 0;
                            while (sorted == 0)
                            {
                                int no_more_sorts = 1;
                                for (long sort = 0; sort < sort_size; sort++)  //--use a bubblesort to sort all the X-coordinates in the new vector
                                {
                                    if (X_temp_list[sort] > X_temp_list[sort+1])
                                    {
                                        no_more_sorts = 0;
                                        //swap
                                        float temp_X = X_temp_list[sort];
                                        int temp_Y = Y_temp_list[sort];
                                        X_temp_list[sort] = X_temp_list[sort+1];
                                        Y_temp_list[sort] = Y_temp_list[sort+1];
                                        X_temp_list[sort+1] = temp_X;
                                        Y_temp_list[sort+1] = temp_Y;
                                    }
                                }
                                if (no_more_sorts == 1)
                                {
                                    sorted = 1;
                                }
                                sort_size--;
                            }

                            //--when sorted, write new line in file, going 0 to n or n to 0, depending on direction.
                            output_data << "G1 Y" << QString::number(grid_line_coordinate[l*12],'f',2) << " Z" << Z_height << " D" << grid_line_direction[l*12] << "\n";
                            //add the nozzle code commands
                            if (grid_line_direction[l*12] == 0)
                            {
                                int *nozzle_active = new int[ui->nozzle_rows_spinbox->value()];
                                for (int n = 0; n < ui->nozzle_rows_spinbox->value(); n++)
                                {
                                    nozzle_active[n] = 0;
                                }
                                for (long e = X_temp_list.size()-1; e >= 0; e--) //negative (D0) sweep
                                {
                                    //see if is on or off, make new toggle value based on this.
                                    int toggle_value;
                                    if (nozzle_active[Y_temp_list[e]%ui->nozzle_rows_spinbox->value()] == 0)
                                    {
                                        toggle_value = 1;
                                        nozzle_active[Y_temp_list[e]%ui->nozzle_rows_spinbox->value()] = 1;
                                    }
                                    else
                                    {
                                        toggle_value = 0;
                                        nozzle_active[Y_temp_list[e]%ui->nozzle_rows_spinbox->value()] = 0;
                                    }

                                    //look ahead when possible
                                    if (e > 1)
                                    {
                                        //round X-coor for checking
                                        QString tempXstring1 = QString::number(X_temp_list[e],'f',2);
                                        float tempXfloat1 = tempXstring1.toFloat();
                                        QString tempXstring2 = QString::number(X_temp_list[e-1],'f',2);
                                        float tempXfloat2 = tempXstring2.toFloat();

                                        //look ahead, if nex X is same
                                        if (tempXfloat1 == tempXfloat2)
                                        {
                                            output_data << "N" << Y_temp_list[e]%ui->nozzle_rows_spinbox->value() << " ";
                                        }
                                        else
                                        {
                                            output_data << "N" << Y_temp_list[e]%ui->nozzle_rows_spinbox->value() << " X" << QString::number(X_temp_list[e],'f',2) << " T" << toggle_value << "\n";
                                        }
                                        //else, write normal line
                                    }
                                    else
                                    {
                                        output_data << "N" << Y_temp_list[e]%ui->nozzle_rows_spinbox->value() << " X" << QString::number(X_temp_list[e],'f',2) << " T" << toggle_value << "\n";
                                    }
                                }
                            }
                            else
                            {
                                int *nozzle_active = new int[ui->nozzle_rows_spinbox->value()];
                                for (int n = 0; n < ui->nozzle_rows_spinbox->value(); n++)
                                {
                                    nozzle_active[n] = 0;
                                }
                                for (long e = 0; e < X_temp_list.size(); e++) //positive (D1) sweep
                                {
                                    //see if is on or off, make new toggle value based on this.
                                    int toggle_value;
                                    if (nozzle_active[Y_temp_list[e]%ui->nozzle_rows_spinbox->value()] == 0)
                                    {
                                        toggle_value = 1;
                                        nozzle_active[Y_temp_list[e]%ui->nozzle_rows_spinbox->value()] = 1;
                                    }
                                    else
                                    {
                                        toggle_value = 0;
                                        nozzle_active[Y_temp_list[e]%ui->nozzle_rows_spinbox->value()] = 0;
                                    }

                                    if (e < X_temp_list.size()-1)
                                    {
                                        //round X-coor for checking
                                        QString tempXstring1 = QString::number(X_temp_list[e],'f',2);
                                        float tempXfloat1 = tempXstring1.toFloat();
                                        QString tempXstring2 = QString::number(X_temp_list[e+1],'f',2);
                                        float tempXfloat2 = tempXstring2.toFloat();

                                        //look ahead, if nex X is same
                                        if (tempXfloat1 == tempXfloat2)
                                        {
                                            output_data << "N" << Y_temp_list[e]%ui->nozzle_rows_spinbox->value() << " ";
                                        }
                                        else
                                        {
                                            output_data << "N" << Y_temp_list[e]%ui->nozzle_rows_spinbox->value() << " X" << QString::number(X_temp_list[e],'f',2) << " T" << toggle_value << "\n";
                                        }
                                        //else, write normal line
                                    }
                                    else
                                    {
                                        output_data << "N" << Y_temp_list[e]%ui->nozzle_rows_spinbox->value() << " X" << QString::number(X_temp_list[e],'f',2) << " T" << toggle_value << "\n";
                                    }
                                }
                            }
                        }
                    }

                    //clear vector for new layer
                    //current height is new height
                    Z_height = conversion_coor_buffer[2];
                    buffer_X_coor.clear();
                    buffer_Y_coor.clear();
                }
                //if E moved forward, printing line, else, travel line
                if (conversion_coor_buffer[3] > coordinate_history[3])
                {
                    if (printing_shape == 0) //mark the start of the shape
                    {
                        printing_shape = 1;
                        for (int c = 0; c < 5; c++)
                        {
                            starting_coordinates[c] = coordinate_history[c];
                        }
                    }
                    //printing line. Draw line from history to current and calculate intersections
                    //determine direction of the line
                    if (conversion_coor_buffer[1] > coordinate_history[1]) //positive Y-direction
                    {
                        //create intersection formula (y = mx+b, but only when X history and X have changed
                        //----------------------fix later to more stable version, now creates different values on straight line
                        if(conversion_coor_buffer[0] == coordinate_history[0])
                        {
                            conversion_coor_buffer[0] = conversion_coor_buffer[0] - 0.0001;
                        }
                        float linem = ((conversion_coor_buffer[1]-coordinate_history[1])/(conversion_coor_buffer[0]-coordinate_history[0]));
                        float pointb = (conversion_coor_buffer[1]-(linem*conversion_coor_buffer[0]));

                        for (int i = 0; i < grid_number_of_lines; i++)
                        {
                            if (grid_line_coordinate[i] > coordinate_history[1] && grid_line_coordinate[i] < conversion_coor_buffer[1])
                            {
                                //calculate X-point and add to array
                                float temp_line_point_calc = ((grid_line_coordinate[i] - pointb)/linem);

                                buffer_X_coor.append(temp_line_point_calc);
                                buffer_Y_coor.append(i);
                            }
                        }
                    }
                    else if (conversion_coor_buffer[1] < coordinate_history[1]) //negative Y-direction
                    {
                        //create intersection formula (y = mx+b)
                        if(conversion_coor_buffer[0] == coordinate_history[0])
                        {
                            conversion_coor_buffer[0] = conversion_coor_buffer[0] - 0.0001;
                        }
                        float linem = ((conversion_coor_buffer[1]-coordinate_history[1])/(conversion_coor_buffer[0]-coordinate_history[0]));
                        float pointb = (conversion_coor_buffer[1]-(linem*conversion_coor_buffer[0]));

                        for (int i = 0; i < grid_number_of_lines; i++)
                        {
                            if (grid_line_coordinate[i] < coordinate_history[1] && grid_line_coordinate[i] > conversion_coor_buffer[1])
                            {
                                //calculate X-point and add to array
                                float temp_line_point_calc = ((grid_line_coordinate[i] - pointb)/linem);

                                buffer_X_coor.append(temp_line_point_calc);
                                buffer_Y_coor.append(i);
                            }
                        }
                    }
                    else //no Y translation, ignore
                    {
                    }

                    //history is current coor
                    for (int c = 0; c < 5; c++)
                    {
                        coordinate_history[c] = conversion_coor_buffer[c];
                    }
                }
                else //when travel and shape end point is close enough to the start point, complete shape
                {
                    //travel line, if shape was printing, try to close shape
                    if ( printing_shape == 1)
                    {
                        printing_shape = 0;

                        //close gap
                        //create intersection formula (y = mx+b)
                        if(starting_coordinates[0] == coordinate_history[0])
                        {
                            starting_coordinates[0] = starting_coordinates[0] - 0.0001;
                        }
                        float linem = ((starting_coordinates[1]-coordinate_history[1])/(starting_coordinates[0]-coordinate_history[0]));
                        float pointb = (starting_coordinates[1]-(linem*starting_coordinates[0]));

                        for (int i = 0; i < grid_number_of_lines; i++)
                        {
                            if (starting_coordinates[1] > coordinate_history[1]) //positive Y-direction
                            {
                                if (grid_line_coordinate[i] > coordinate_history[1] && grid_line_coordinate[i] < starting_coordinates[1])
                                {
                                    //calculate X-point and add to array
                                    float temp_line_point_calc = ((grid_line_coordinate[i] - pointb)/linem);

                                    buffer_X_coor.append(temp_line_point_calc);
                                    buffer_Y_coor.append(i);
                                }
                            }
                            else if (starting_coordinates[1] < coordinate_history[1]) //negative Y-direction
                            {
                                if (grid_line_coordinate[i] < coordinate_history[1] && grid_line_coordinate[i] > starting_coordinates[1])
                                {
                                    //calculate X-point and add to array
                                    float temp_line_point_calc = ((grid_line_coordinate[i] - pointb)/linem);

                                    buffer_X_coor.append(temp_line_point_calc);
                                    buffer_Y_coor.append(i);
                                }
                            }
                        }
                        //reset starting coordinates

                    }

                    //gap too big, give error and close anyway

                    //if shape was traveling, simply write to history
                    for (int c = 0; c < 5; c++)
                    {
                        coordinate_history[c] = conversion_coor_buffer[c];
                    }
                }
            }



            if (readtextStream.atEnd())
            {
                //reorder vector
                for (int l = 0; l < (grid_number_of_lines/ui->nozzle_rows_spinbox->value()); l++)
                {
                    //--create lines read integer
                    //--for loop through the file, marking multiples of nozzlecount, ie. 0-11, 12-23 etc.
                    int value_read = 0;
                    int lower_limit = l * ui->nozzle_rows_spinbox->value();
                    int upper_limit = lower_limit + ui->nozzle_rows_spinbox->value();
                    QVector<float> X_temp_list(0);
                    QVector<int> Y_temp_list(0);
                    for (int n = 0; n < buffer_Y_coor.size(); n++)
                    {
                        if (buffer_Y_coor[n] >= lower_limit && buffer_Y_coor[n] < upper_limit)  //--when found, move these to a new vector
                        {
                            value_read = 1;
                            X_temp_list.append(buffer_X_coor[n]);
                            Y_temp_list.append(buffer_Y_coor[n]);
                        }
                    }
                    if (value_read == 1) //if a value has been read, convert line
                    {
                        long sort_size = X_temp_list.size() - 1;
                        int sorted = 0;
                        while (sorted == 0)
                        {
                            int no_more_sorts = 1;
                            for (long sort = 0; sort < sort_size; sort++)  //--use a bubblesort to sort all the X-coordinates in the new vector
                            {
                                if (X_temp_list[sort] > X_temp_list[sort+1])
                                {
                                    no_more_sorts = 0;
                                    //swap
                                    float temp_X = X_temp_list[sort];
                                    int temp_Y = Y_temp_list[sort];
                                    X_temp_list[sort] = X_temp_list[sort+1];
                                    Y_temp_list[sort] = Y_temp_list[sort+1];
                                    X_temp_list[sort+1] = temp_X;
                                    Y_temp_list[sort+1] = temp_Y;
                                }
                            }
                            if (no_more_sorts == 1)
                            {
                                sorted = 1;
                            }
                            sort_size--;
                        }

                        //--when sorted, write new line in file, going 0 to n or n to 0, depending on direction.
                        output_data << "G1 Y" << grid_line_coordinate[l*12] << " Z" << Z_height << " D" << grid_line_direction[l*12] << "\n";
                        //add the nozzle code commands
                        if (grid_line_direction[l*12] == 0)
                        {
                            int *nozzle_active = new int[ui->nozzle_rows_spinbox->value()];
                            for (int n = 0; n < ui->nozzle_rows_spinbox->value(); n++)
                            {
                                nozzle_active[n] = 0;
                            }
                            for (long e = X_temp_list.size()-1; e >= 0; e--) //negative (D0) sweep
                            {
                                //see if is on or off, make new toggle value based on this.
                                int toggle_value;
                                if (nozzle_active[Y_temp_list[e]%ui->nozzle_rows_spinbox->value()] == 0)
                                {
                                    toggle_value = 1;
                                    nozzle_active[Y_temp_list[e]%ui->nozzle_rows_spinbox->value()] = 1;
                                }
                                else
                                {
                                    toggle_value = 0;
                                    nozzle_active[Y_temp_list[e]%ui->nozzle_rows_spinbox->value()] = 0;
                                }

                                //look ahead when possible
                                if (e > 1)
                                {
                                    //round X-coor for checking
                                    QString tempXstring1 = QString::number(X_temp_list[e],'f',2);
                                    float tempXfloat1 = tempXstring1.toFloat();
                                    QString tempXstring2 = QString::number(X_temp_list[e-1],'f',2);
                                    float tempXfloat2 = tempXstring2.toFloat();

                                    //look ahead, if nex X is same
                                    if (tempXfloat1 == tempXfloat2)
                                    {
                                        output_data << "N" << Y_temp_list[e]%ui->nozzle_rows_spinbox->value() << " ";
                                    }
                                    else
                                    {
                                        output_data << "N" << Y_temp_list[e]%ui->nozzle_rows_spinbox->value() << " X" << QString::number(X_temp_list[e],'f',2) << " T" << toggle_value << "\n";
                                    }
                                    //else, write normal line
                                }
                                else
                                {
                                    output_data << "N" << Y_temp_list[e]%ui->nozzle_rows_spinbox->value() << " X" << QString::number(X_temp_list[e],'f',2) << " T" << toggle_value << "\n";
                                }
                            }
                        }
                        else
                        {
                            int *nozzle_active = new int[ui->nozzle_rows_spinbox->value()];
                            for (int n = 0; n < ui->nozzle_rows_spinbox->value(); n++)
                            {
                                nozzle_active[n] = 0;
                            }
                            for (long e = 0; e < X_temp_list.size(); e++) //positive (D1) sweep
                            {
                                //see if is on or off, make new toggle value based on this.
                                int toggle_value;
                                if (nozzle_active[Y_temp_list[e]%ui->nozzle_rows_spinbox->value()] == 0)
                                {
                                    toggle_value = 1;
                                    nozzle_active[Y_temp_list[e]%ui->nozzle_rows_spinbox->value()] = 1;
                                }
                                else
                                {
                                    toggle_value = 0;
                                    nozzle_active[Y_temp_list[e]%ui->nozzle_rows_spinbox->value()] = 0;
                                }

                                if (e < X_temp_list.size()-1)
                                {
                                    //round X-coor for checking
                                    QString tempXstring1 = QString::number(X_temp_list[e],'f',2);
                                    float tempXfloat1 = tempXstring1.toFloat();
                                    QString tempXstring2 = QString::number(X_temp_list[e+1],'f',2);
                                    float tempXfloat2 = tempXstring2.toFloat();

                                    //look ahead, if nex X is same
                                    if (tempXfloat1 == tempXfloat2)
                                    {
                                        output_data << "N" << Y_temp_list[e]%ui->nozzle_rows_spinbox->value() << " ";
                                    }
                                    else
                                    {
                                        output_data << "N" << Y_temp_list[e]%ui->nozzle_rows_spinbox->value() << " X" << QString::number(X_temp_list[e],'f',2) << " T" << toggle_value << "\n";
                                    }
                                    //else, write normal line
                                }
                                else
                                {
                                    output_data << "N" << Y_temp_list[e]%ui->nozzle_rows_spinbox->value() << " X" << QString::number(X_temp_list[e],'f',2) << " T" << toggle_value << "\n";
                                }
                            }
                        }
                    }
                }
                end_of_file = 1; //end conversion
            }
            //end file
        }
        OutputFile.close(); //when done, close the file
        Inputfile.close();
    }
}


//ui->Debug_label->setText(QString::number((grid_number_of_lines)));

/*QMessageBox messagetemp;
messagetemp.setText(decode_line[character_reading]);
messagetemp.exec();*/

/*
 QMessageBox messagetemp;
messagetemp.setText(QString::number(linem));
messagetemp.exec();*/

void MainWindow::on_Test_button_clicked()
{
    QVector<float> testvector(0);
    for (int i = 0; i < 300; i++)
    {
        testvector.append(1.00);
    }



    QMessageBox messagetemp;
    messagetemp.setText(QString::number(testvector.size()));
    messagetemp.exec();
}
